@@ -1,6 +1,7 @@
 #include <config.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-macros.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #if HAVE_EXIF
 #include <libexif/exif-entry.h>
 #include <libexif/exif-data.h>
@@ -8,11 +9,17 @@
 
 #include "eog-info-view.h"
 
+enum {
+	SIGNAL_ID_SIZE_PREPARED,
+	SIGNAL_ID_INFO_FINISHED,
+	SIGNAL_ID_CHANGED,
+	SIGNAL_ID_LAST
+};
+
 struct _EogInfoViewPrivate
 {
 	EogImage *image;
-	int image_cb_id;
-	int image_cb_id2;
+	int image_signal_ids [SIGNAL_ID_LAST];
 };
 
 GNOME_CLASS_BOILERPLATE (EogInfoView, eog_info_view,
@@ -63,13 +70,15 @@ eog_info_view_instance_init (EogInfoView *view)
 	EogInfoViewPrivate *priv;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *cell;
+	int s;
 
 	priv = g_new0 (EogInfoViewPrivate, 1);
 
 	view->priv = priv;
 	priv->image = NULL;
-	priv->image_cb_id = 0;
-	priv->image_cb_id2 = 0;
+	for (s = 0; s < SIGNAL_ID_LAST; s++) {
+		priv->image_signal_ids [s] = 0;
+	} 
 
 	/* tag column */
 	cell = gtk_cell_renderer_text_new ();
@@ -123,6 +132,74 @@ append_row (GtkListStore *store, const char *attribute, const char *value)
 	}
 }
 
+static void
+add_image_size_attribute (EogInfoView *view, EogImage *image, GtkListStore *store)
+{
+	char buffer[32];
+	GtkTreeIter iter;
+	int width, height;
+	GnomeVFSFileSize bytes;
+	char *size_str;
+
+	g_return_if_fail (EOG_IS_IMAGE (image));
+	g_return_if_fail (EOG_IS_INFO_VIEW (view));
+	g_return_if_fail (GTK_IS_LIST_STORE (store));
+
+	eog_image_get_size (image, &width, &height);
+	bytes = eog_image_get_bytes (image);
+
+	if (width > -1) {
+		g_snprintf (buffer, 32, "%i", width);	
+	}
+	else {
+		buffer[0] = '\0';
+	}
+
+	if (!gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, "1")) {
+		append_row (store, _("Width"), buffer);
+	}
+	else {
+		gtk_list_store_set (store, &iter, 1, buffer, -1);
+	}
+
+	if (height > -1) {
+		g_snprintf (buffer, 32, "%i", height);	
+	}
+	else {
+		buffer[0] = '\0';
+	}
+
+	if (!gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, "2")) {
+		append_row (store, _("Height"), buffer);
+	}
+	else {
+		gtk_list_store_set (store, &iter, 1, buffer, -1);
+	}
+
+	size_str = gnome_vfs_format_file_size_for_display (bytes);
+	if (!gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, "3")) {
+		append_row (store, _("Filesize"), size_str);
+	}
+	else {
+		gtk_list_store_set (store, &iter, 1, size_str, -1);
+	}
+	g_free (size_str);
+}
+
+static void
+add_image_filename_attribute (EogInfoView *view, EogImage *image, GtkListStore *store)
+{
+	char *caption;
+
+	g_return_if_fail (EOG_IS_IMAGE (image));
+	g_return_if_fail (EOG_IS_INFO_VIEW (view));
+	g_return_if_fail (GTK_IS_LIST_STORE (store));
+
+	caption = eog_image_get_caption (image);
+	append_row (store, _("Filename"), caption);
+}
+
+
 #if HAVE_EXIF
 static void 
 exif_entry_cb (ExifEntry *entry, gpointer data)
@@ -147,39 +224,26 @@ fill_list_exif (EogImage *image, GtkListStore *store, ExifData *ed)
 }
 #endif
 
+/*  add_image_exif_attribute
+ *
+ *  This function will only add information additional to filename,
+ *  image width and image height. Main purpose is to present
+ *  EXIF information.
+ */
 static void
-fill_list (EogInfoView *view)
+add_image_exif_attribute (EogInfoView *view, EogImage *image, GtkListStore *store)
 {
-	GtkListStore *store;
-	EogImage *image;
-	char buffer[32];
-	int width, height;
 #if HAVE_EXIF
 	ExifData *ed;
 #endif
-	if (gtk_tree_view_get_model (GTK_TREE_VIEW (view)) == NULL) return;
 
-	store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (view)));
-	image = view->priv->image;
+	g_return_if_fail (EOG_IS_IMAGE (image));
+	g_return_if_fail (EOG_IS_INFO_VIEW (view));
+	g_return_if_fail (GTK_IS_LIST_STORE (store));
 
-	g_assert (image != NULL);
-
-	eog_image_get_size (image, &width, &height);
-	if (width > -1) {
-		g_snprintf (buffer, 32, "%i", width);	
-		append_row (store, _("Width"), buffer);
-	}
-	else {
-		append_row (store, _("Width"), NULL);
-	}
-
-	if (height > -1) {
-		g_snprintf (buffer, 32, "%i", height);
-		append_row (store, _("Height"), buffer);
-	}
-	else {
-		append_row (store, _("Height"), NULL);
-	}
+	/* add further information to the info view list here, which are not EXIF dependend 
+	 * (e.g. file size) 
+	 */
 
 #if HAVE_EXIF
 	ed = (ExifData*) eog_image_get_exif_information (image);
@@ -190,40 +254,70 @@ fill_list (EogInfoView *view)
 #endif
 }
 
+/* loading_size_prepared_cb
+ *
+ * This function is always called, when the image dimension is known. Therefore
+ * we add the image size and width to the info view list here.
+ */
 static void
 loading_size_prepared_cb (EogImage *image, gint width, gint height, gpointer data)
 {
 	GtkListStore *store;
-	char buffer[32];
-	GtkTreeIter iter;
-	
-	EogInfoView *view = EOG_INFO_VIEW (data);
+	EogInfoView *view;
+
+	view = EOG_INFO_VIEW (data);
 
 	if (gtk_tree_view_get_model (GTK_TREE_VIEW (view)) == NULL) return;
 
 	store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (view)));
 
-	g_snprintf (buffer, 32, "%i", width);	
-	if (!gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, "1")) {
-		append_row (store, _("Width"), buffer);
-	}
-	else {
-		gtk_list_store_set (store, &iter, 1, buffer, -1);
-	}
-
-	g_snprintf (buffer, 32, "%i", height);	
-	if (!gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (store), &iter, "2")) {
-		append_row (store, _("Height"), buffer);
-	}
-	else {
-		gtk_list_store_set (store, &iter, 1, buffer, -1);
-	}
+	add_image_size_attribute (view, image, store);
 }
 
+/* loading_info_finished_cb
+ * 
+ * This function is called if EXIF data has been read by EogImage.
+ */
 static void
 loading_info_finished_cb (EogImage *image, gpointer data)
 {
-	fill_list (EOG_INFO_VIEW (data));
+	EogInfoView *view;
+	GtkListStore *store;
+
+	g_return_if_fail (EOG_IS_IMAGE (image));
+
+	view = EOG_INFO_VIEW (data);
+	store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (view)));
+
+	add_image_size_attribute (view, image, store);
+	add_image_exif_attribute (view, image, store);
+}
+
+
+/* image_changed_cb
+ *
+ * This function is called every time the image changed somehow. We
+ * will clear everything and rewrite all attributes.
+ */
+static void
+image_changed_cb (EogImage *image, gpointer data)
+{
+	EogInfoView *view;
+	GtkListStore *store;
+
+	g_return_if_fail (EOG_IS_IMAGE (image));
+
+	view = EOG_INFO_VIEW (data);
+
+	/* create new list */
+	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+
+	/* add image attributes */
+	add_image_filename_attribute (view, image, store);
+	add_image_size_attribute (view, image, store);
+	add_image_exif_attribute (view, image, store);
+
+	gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (store));
 }
 
 void
@@ -231,7 +325,7 @@ eog_info_view_set_image (EogInfoView *view, EogImage *image)
 {
 	EogInfoViewPrivate *priv;
 	GtkListStore *store;
-	char *caption;
+	int s;
 
 	g_return_if_fail (EOG_IS_INFO_VIEW (view));
 
@@ -241,13 +335,11 @@ eog_info_view_set_image (EogInfoView *view, EogImage *image)
 		return;
 
 	if (priv->image != NULL) {
-		if (priv->image_cb_id > 0) {
-			g_signal_handler_disconnect (G_OBJECT (priv->image), priv->image_cb_id);
-			priv->image_cb_id = 0;
-		}
-		if (priv->image_cb_id2 > 0) {
-			g_signal_handler_disconnect (G_OBJECT (priv->image), priv->image_cb_id2);
-			priv->image_cb_id2 = 0;
+		for (s = 0; s < SIGNAL_ID_LAST; s++) {
+			if (priv->image_signal_ids [s] > 0) {
+				g_signal_handler_disconnect (G_OBJECT (priv->image), priv->image_signal_ids [s]);
+				priv->image_signal_ids [s] = 0;
+			}
 		}
 
 		g_object_unref (priv->image);
@@ -265,15 +357,24 @@ eog_info_view_set_image (EogInfoView *view, EogImage *image)
 	priv->image = image;
 
 	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
-	caption = eog_image_get_caption (image);
-	append_row (store, _("Filename"), caption);
+	/* display at least the filename */
+	add_image_filename_attribute (view, image, store);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (store));
 
-	priv->image_cb_id = g_signal_connect (G_OBJECT (priv->image), "loading_info_finished", 
-					      G_CALLBACK (loading_info_finished_cb), view);
-	priv->image_cb_id2 = g_signal_connect (G_OBJECT (priv->image), "loading_size_prepared", 
-					      G_CALLBACK (loading_size_prepared_cb), view);
+	/* prepare additional image information callbacks */
+	priv->image_signal_ids [SIGNAL_ID_INFO_FINISHED] = 
+		g_signal_connect (G_OBJECT (priv->image), "loading_info_finished", 
+				  G_CALLBACK (loading_info_finished_cb), view);
 
+	priv->image_signal_ids [SIGNAL_ID_SIZE_PREPARED] = 
+		g_signal_connect (G_OBJECT (priv->image), "loading_size_prepared", 
+				  G_CALLBACK (loading_size_prepared_cb), view);
+
+	priv->image_signal_ids [SIGNAL_ID_CHANGED] = 
+		g_signal_connect (G_OBJECT (priv->image), "image_changed", 
+				  G_CALLBACK (image_changed_cb), view);
+
+	/* start loading */
 	eog_image_load (priv->image, EOG_IMAGE_LOAD_DEFAULT);
 }
 
