@@ -13,8 +13,9 @@
 #include "eog-hig-dialog.h"
 #include "eog-window.h"
 #include "session.h"
+#include "eog-config-keys.h"
 
-static void open_uri_list_cb (EogWindow *window, GList *uri_list, gpointer data); 
+static void open_uri_list_cb (EogWindow *window, GSList *uri_list, gpointer data); 
 static GtkWidget* create_new_window (void);
 
 
@@ -75,13 +76,17 @@ open_window (LoadContext *ctx)
 	GError *error = NULL;
 	gboolean new_window;
 	GList *it;
+	GConfClient *client;
 
 	g_return_val_if_fail (ctx->iid != NULL, FALSE);
 
-	/* FIXME: load gconf key */
-	new_window = TRUE; /* gconf_client_get_bool (priv->client, "/apps/eog/window/open_new_window", NULL); */
+	client = gconf_client_get_default ();
+
+	new_window = gconf_client_get_bool (client, EOG_CONF_WINDOW_OPEN_NEW_WINDOW, NULL);
 	new_window = new_window && ((ctx->window == NULL) || eog_window_has_contents (ctx->window));
 
+	g_object_unref (client);
+
 	if (ctx->single_windows) 
 	{
 		for (it = ctx->uri_list; it != NULL; it = it->next) {
@@ -161,9 +166,9 @@ make_canonical_uri (const char *path)
  * regular files and one list with all directories.
  **/
 static void
-sort_files (GList *files, GList **file_list, GList **dir_list, GList **error_list)
+sort_files (GSList *files, GList **file_list, GList **dir_list, GList **error_list)
 {
-	GList *it;
+	GSList *it;
 	GnomeVFSFileInfo *info;
 	
 	info = gnome_vfs_file_info_new ();
@@ -297,18 +302,22 @@ show_nonexistent_files (GList *error_list)
 	char *str;
 	int n;
 	GtkWidget *dlg;
+	char *path;
 
 	g_assert (error_list != NULL);
 
 	str = concat_string_list_with_newlines (error_list, &n);
 
-	dlg = eog_hig_dialog_new (GTK_STOCK_DIALOG_ERROR, _("Could not find files"), str, TRUE);
+	path = gnome_vfs_format_uri_for_display (str);
+
+	dlg = eog_hig_dialog_new (GTK_STOCK_DIALOG_ERROR, _("Could not find files"), path, TRUE);
 	gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
 	
 	gtk_widget_show (dlg);
 	gtk_dialog_run (GTK_DIALOG (dlg));
 	gtk_widget_destroy (dlg);
 
+	g_free (path);
 	g_free (str);
 }
 
@@ -328,7 +337,7 @@ string_array_to_list (const gchar **files)
 }
 
 static void 
-open_uri_list_cb (EogWindow *window, GList *uri_list, gpointer data)
+open_uri_list_cb (EogWindow *window, GSList *uri_list, gpointer data)
 {
 	GList *file_list = NULL;
 	GList *dir_list = NULL;
